<?php
/**
 * A class for directory reading and getting its contents, no recursion!
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2013 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FrameworkDirectoryPointer extends BaseFrameworkSystem {
	/**
	 * The current path we are working in
	 */
	private $pathName = '';

	/**
	 * The directory iterator instance
	 */
	private $directoryInstance = NULL;

	/**
	 * Protected constructor
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Destructor for cleaning purposes, etc
	 */
	public function __destruct() {
		// Is there a resource pointer? Then we have to close the directory here!
		if ($this->getDirectoryInstance() instanceof DirectoryIterator) {
			// Try to close a directory
			$this->closeDirectory();
		} // END - if

		// Call the parent destructor
		parent::__destruct();
	}

	/**
	 * Create a directory pointer based on the given path. The path will also
	 * be verified here.
	 *
	 * @param	$pathName		The path name we shall pass to opendir()
	 * @param	$inConstructor	If we are in de/con-structor or from somewhere else
	 * @return	$pointerInstance	A prepared instance of FrameworkDirectoryPointer
	 * @throws	PathIsEmptyException	If the provided path name is empty
	 * @throws	InvalidPathStringException	If the provided path name is not a string
	 * @throws	PathIsNoDirectoryException	If the provided path name is not valid
	 * @throws	PathReadProtectedException	If the provided path name is read-protected
	 * @todo	Get rid of inConstructor, could be old-lost code.
	 */
	public static final function createFrameworkDirectoryPointer ($pathName, $inConstructor = FALSE) {
		// Some pre-sanity checks...
		if (is_null($pathName)) {
			// No pathname given
			if ($inConstructor) {
				return NULL;
			} else {
				throw new PathIsEmptyException(NULL, self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
			}
		} elseif (!is_string($pathName)) {
			// Is not a string
			if ($inConstructor) {
				return NULL;
			} else {
				throw new InvalidPathStringException(NULL, self::EXCEPTION_INVALID_STRING);
			}
		} elseif (!is_dir($pathName)) {
			// Not a directory
			if ($inConstructor) {
				return NULL;
			} else {
				throw new PathIsNoDirectoryException($pathName, self::EXCEPTION_INVALID_PATH_NAME);
			}
		} elseif (!is_readable($pathName)) {
			// Not readable
			if ($inConstructor) {
				return NULL;
			} else {
				throw new PathReadProtectedException($pathName, self::EXCEPTION_READ_PROTECED_PATH);
			}
		}

		// Create new instance
		$pointerInstance = new FrameworkDirectoryPointer();

		// Get an iterator for the directory
		$directoryInstance = new DirectoryIterator($pathName);

		// Set directory pointer and path name
		$pointerInstance->setDirectoryInstance($directoryInstance);
		$pointerInstance->setPathName($pathName);

		// Return the instance
		return $pointerInstance;
	}

	/**
	 * Read raw lines of data from a directory pointer and return the data
	 *
	 * @return	$current	Current entry from encapsulated iterator
	 */
	public function readRawDirectory () {
		// Can the next entry be read?
		assert($this->getDirectoryInstance()->valid());

		// Default is FALSE
		$current = FALSE;

		// Is it a dot directory?
		if (!$this->getDirectoryInstance()->isDot()) {
			// Read data from the directory pointer and return it
			$current = $this->getDirectoryInstance()->current();
		} // END - if

		// Advance to next entry
		$this->getDirectoryInstance()->next();

		// Return found entry
		return $current;
	}

	/**
	 * Read lines from the current directory pointer except some parts
	 *
	 * @param	$except		Some parts of a directory we want to ignore. Valid: directory and file names, other values will be silently ignored
	 * @return	string		Directory and/or file names read from the current directory pointer
	 */
	public function readDirectoryExcept (array $except = array()) {
		// No exceptions given?
		if (count($except) == 0) {
			// No exception given, so read all files and directories, but not recursive
			self::createDebugInstance(__CLASS__)->debugOutput('DIRECTORY[' . __METHOD__ . ':' . __LINE__ . ']: No exceptions given, please use readRawDirectory() instead!');
			return $this->readRawDirectory();
		} // END - if

		// Read a raw line...
		$rawLine = $this->readRawDirectory();
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DIRECTORY[' . __METHOD__ . ':' . __LINE__ . ']: rawLine[' . gettype($rawLine) . ']=' . $rawLine);

		// Shall we exclude directories?
		if ((!is_null($rawLine)) && ($rawLine !== FALSE) && (!in_array($rawLine, $except))) {
			// Return read data
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DIRECTORY[' . __METHOD__ . ':' . __LINE__ . ']: rawLine[' . gettype($rawLine) . ']=' . $rawLine);
			return $rawLine;
		} elseif ((!is_null($rawLine)) && ($rawLine !== FALSE)) {
			// Exclude this part
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DIRECTORY[' . __METHOD__ . ':' . __LINE__ . ']: rawline[' . gettype($rawLine) . ']=' . $rawLine . ' - Recursive call!');
			return $this->readDirectoryExcept($except);
		}

		// End pointer reached
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DIRECTORY[' . __METHOD__ . ':' . __LINE__ . ']: Returning NULL!');
		return NULL;
	}

	/**
	 * Close a directory source and set it's instance to null and the path name
	 * to empty
	 *
	 * @return	void
	 */
	public function closeDirectory () {
		// Close the directory by unsetting it
		$this->setDirectoryInstance(NULL);
		$this->setPathName('');
	}

	/**
	 * Setter for the directory pointer
	 *
	 * @param	$directoryInstance	An instanceof a DirectoryIterator class or NULL to unset ("close") it.
	 * @return	void
	 */
	protected final function setDirectoryInstance (DirectoryIterator $directoryInstance = NULL) {
		// Set instance (or NULL)
		$this->directoryInstance = $directoryInstance;
	}

	/**
	 * Getter for the directory pointer
	 *
	 * @return	$directoryInstance	The directory pointer which shall be a valid directory resource
	 */
	public final function getDirectoryInstance () {
		return $this->directoryInstance;
	}

	/**
	 * Setter for path name
	 *
	 * @param	$pathName	The new path name
	 * @return	void
	 */
	public final function setPathName ($pathName) {
		$pathName = (string) $pathName;
		$this->pathName = $pathName;
	}

	/**
	 * Getter for path name
	 *
	 * @return	$pathName	The current path name
	 */
	public final function getPathName () {
		return $this->pathName;
	}
}

// [EOF]
?>
